//same colors as on previous charts
window.chartColors = ["#edc240", "#afd8f8", "#cb4b4b", "#4da74d", "#9440ed"];
//for dashboard bars
window.lightGreen = "#95bb31";

(function () {
  moment.locale('en');
  var charts = {};

  /**
   * Create a chart
   */
  var createChart = function(type){

    var chart = {
      plot: null,
      seriesOpts: {},
      from: LS.chart.seriesConf.from,
      fromTXT: moment(new Date(LS.chart.seriesConf.from)).format('YYYY-MM-DD'),
      to: LS.chart.seriesConf.to,
      toTXT: moment(new Date(LS.chart.seriesConf.to)).format('YYYY-MM-DD'),
      interval: LS.chart.seriesConf.interval,
      type: type,
      useBars: LS.chart.useBars[type],
      url: LS.chart.dataUrl,
      csvUrl: LS.chart.csvUrl,
      seriesData: LS.chart.seriesData[type],
      totalTokenId: LS.chart.seriesConf.totalTokenId,

      /**
       * Initialization method. Called on chart creation
       */
      init: function(){
        var that = this;
        this.seriesOpts = {
          options: {
            legend: {
              display: false
            },
            tooltips: {
              enabled: true,
              displayColors: false,
              callbacks: {
                //show all data in tooltip header
                title: function (tooltipItem) {
                  return moment(tooltipItem[0].xLabel).format('MMM D') + " - " + that.createValueLabel(tooltipItem[0].yLabel);
                },
                //Disable default way to show tooltip
                label: function () {
                  return null;
                }
              }
            },
            scales: {
              xAxes: [{
                type: 'time',
                time: {
                  minUnit: 'day',
                  round: 'day'
                },
                gridLines: {
                  //make zeroline be of the same color as others, otherwise first grid line is bolder
                  zeroLineColor: 'rgba(0, 0, 0, 0.1)'
                }
              }],
              yAxes: [{
                ticks: {
                  beginAtZero:true,
                  min: 0,
                  callback: function(value, index, values) {
                    //filter out non integer numbers
                    //todo may be there is better check?
                    if (Math.floor(value) === value) {
                      return that.yTickLabel(value, index, values);
                    }
                  }
                }
              }]
            }
          },
          data: {
            datasets: []
          },
          type: 'line'
        };
        this.timesavedChart = that.type.toLowerCase().indexOf('timesaved') !== -1;
        if(this.timesavedChart) {
          this.convertValue = function (value) {
            return moment.duration(value).asHours();
          };
          this.yTickLabel = function(value) {
            return value + "h";
          };
          this.createValueLabel = function (hours) {
            var flooredHours = Math.floor(hours);
            if (flooredHours === 0) {
              return Math.floor(hours * 60) + " minutes saved";
            }
            else if (flooredHours === 1) {
              return flooredHours + " hour saved";
            }
            return flooredHours + " hours saved";
          };
        }
        //TODO refactor creation to separate methods to be less messy.
        //TODO Basically there will be 2 types of charts bars timesaved and line seats in use. Plus dougnuts later.
        if (this.useBars) {
          this.seriesOpts.type = 'bar';
          this.seriesOpts.options.scales.xAxes = [{type: 'category'}];
          //bar chart is only on dashboard now and has no interaction, so this is valid.
          var tempDataset = this.seriesData.map(function (line) {
            return that.normalizeToChartDataset(line);
          });
          that.seriesOpts.options.scales.xAxes[0].gridLines = {
            display: false
          };
          that.seriesOpts.data.labels = tempDataset[0].data.map(function (point) {
            return moment(point.x).format('MMM D');
          });
          that.seriesOpts.data.datasets = [
            {
              label: tempDataset.label,
              borderColor: window.lightGreen,
              backgroundColor: window.lightGreen,
              data: tempDataset[0].data.map(function (point) {
                return point.y;
              })
            }
          ];
          if (LS.chart.dashboard) {
            //disabling assign colors for dashboard bar chart
            this.assignColors = function() {};
          }
        } else {
          that.seriesOpts.data.datasets = this.seriesData.map(function (line) {
            return that.normalizeToChartDataset(line);
          });
          //workaround to set max yAxis a bit above max value (problem for small numbers, line goes directly on up bound)
          var values = that.seriesOpts.data.datasets.map(function (dataset) {
            return dataset.data
              .map(function (point) { return point.y;})
              .reduce(function(a,b) { return Math.max(a,b);});
          });
          var maxValue = values.reduce(function(a,b) { return Math.max(a,b);});
          this.seriesOpts.options.scales.yAxes[0].ticks.suggestedMax = Math.max(maxValue * 1.05, 1);
        }

        // associate it with either 'chart-timeSaved' or 'chart-seatsInUse' element.
        var ctx = document.getElementById("chart-" + type);
        this.assignColors();
        this.plot = new Chart(ctx, this.seriesOpts);
      },
      convertValue: function (value) {
        return value;
      },
      createValueLabel: function (value) {
        return value;
      },
      yTickLabel: function(value) {
        return value;
      },
      normalizeToChartDataset: function(seriesData) {
        //fill empty values, truncate Xs to beginning of interval
        //todo MAX: verify that we still need it here, probably not, it is done on server
        var tick = this.getTruncatedTick();
        var c = 0;
        var values = [];
        var toDate = new Date(this.to);
        while (tick <= toDate) {
          var value = 0;
          if (seriesData.data[c] && seriesData.data[c][0] <= tick.getTime()) {
            value = seriesData.data[c][1];
            c++;
          }
          values.push({x:tick.getTime(), y:this.convertValue(value)});
          tick = this.getNextTick(tick);
        }

        if (values.length === 1) {
          //if we have only one data point it will be shown on the left border by default which does not look nice.
          //we want it to appear in the middle, so we add two invisible fake points (before and after)
          var postTick = tick;
          var prevTick = this.getNextTick(this.getTruncatedTick(), -1);
          values = [{x: prevTick.getTime(), y:null}, values[0], {x: postTick.getTime(), y:null}];
        }

        return {
          label: seriesData.label,
          data: values,
          fill: false
        };
      },
      getTruncatedTick: function() {
        //todo change to moment.js
        var tick = new Date(this.from);
        if (this.interval === 12) {      // MINUTE
          return new Date(tick.getFullYear(), tick.getMonth(), tick.getDate(), tick.getHours(), tick.getMinutes(), 0, 0);
        }
        else if (this.interval === 11) { // HOUR_OF_DAY
          return new Date(tick.getFullYear(), tick.getMonth(), tick.getDate(), tick.getHours(), 0, 0, 0);
        }
        else if (this.interval === 5) { // DAY_OF_MONTH
          return new Date(tick.getFullYear(), tick.getMonth(), tick.getDate(), tick.getHours(), 0, 0, 0);
        }
        else if (this.interval === 2) { // MONTH
          return new Date(tick.getFullYear(), tick.getMonth(), 1, 0, 0, 0, 0);
        }
        else if (this.interval === 1) { // YEAR
          return new Date(tick.getFullYear(), 0, 1, 0, 0, 0, 0);
        }
        return tick;
      },
      getNextTick: function(date, increment) {
        if (!increment) {
          increment = 1;
        }
        //todo change to moment.js
        if (this.interval === 12) {      // MINUTE
          date.setMinutes(date.getMinutes() + increment);
        }
        else if (this.interval === 11) { // HOUR_OF_DAY
          date.setHours(date.getHours() + increment);
        }
        else if (this.interval === 5) {  // DAY_OF_MONTH
          date.setDate(date.getDate() + increment);
        }
        else if (this.interval === 2) {  // MONTH
          date.setMonth(date.getMonth() + increment);
        }
        else if (this.interval === 1) {  // YEAR
          date.setFullYear(date.getFullYear() + increment);
        }
        return date;
      },
      fullUrl: function(idsList){
        return this.url + '?' + $.param({
          ids: idsList.join(","),
          interval: this.interval,
          from: this.fromTXT,
          to: this.toTXT
        });
      },
      fullCsvUrl: function(){
        return this.csvUrl + '?' + $.param({
          ids: _.map(chart.plot.data.datasets, function(serie){
            return serie.label;
          }).join(","),
          interval: this.interval,
          from: this.fromTXT,
          to: this.toTXT
        });
      },
      // Reset the charts object (draw it again)
      reset: function() {
        var that = this;
        var ids = _.map(chart.plot.data.datasets, function(serie){
          return serie.label;
        });
        $.getJSON(this.fullUrl(ids), function (response) {
          if(that.interval === 2){
            that.seriesOpts.options.scales.xAxes[0].time.minUnit = 'month';
          }else if(that.interval === 1) {
            that.seriesOpts.options.scales.xAxes[0].time.minUnit = 'year';
          }else if (that.interval === 5) {
            that.seriesOpts.options.scales.xAxes[0].time.minUnit = 'day';
          }else{
            that.seriesOpts.options.scales.xAxes[0].time.minUnit = 'minute';
          }
          that.seriesOpts.data.datasets = response.map(function (line) {
            return that.normalizeToChartDataset(line);
          });

          that.assignColors();
          that.plot.update();
        });
      },
      assignColors: function() {
        var series = this.seriesOpts.data.datasets;
        $("#legend-" + this.type + " .ls-chart-legend-item .ls-chart-legend-color").css("background", "lightgrey");
        for (var i = 0; i < series.length; ++i){
          var legendItem = $("#legend-" + this.type + "-item-" + series[i].label);
          var colorId = legendItem.data("color");
          var color = chartColors[colorId % chartColors.length];
          series[i].borderColor = color;
          series[i].backgroundColor = color;
          legendItem.find(".ls-chart-legend-color").css("background", color);
        }
      },
      downloadCsv: function(){
        window.open(this.fullCsvUrl());
      },
      toggleTokenChart: function(id){
        var data = this.seriesOpts.data.datasets;
        var that = this;
        for (var i = 0; i < data.length; i++) {
          if (data[i].label === id) {
            data.splice(i, 1);
            that.assignColors();
            that.plot.update();
            return;
          }
        }

        $.getJSON(chart.fullUrl([id]), function (response) {
          var data = that.seriesOpts.data.datasets;
          data.push(chart.normalizeToChartDataset(response[0]));
          that.assignColors();
          that.plot.update();
        });
      }
    };
    charts[type] = chart;
    return chart;
  };

  // HTML attribute id chart-timeSaved or chart-seatsInUse. They are for example present on the dashboard.
  // Does this code replace those placeholders..?
  if(LS.chart.types.length === 1 && $("#chart").length > 0) {
    $("#chart").attr("id", "chart-" + LS.chart.types[0]);
  }

  // For each chart-type, create the chart
  _.each(LS.chart.types, function(type){
    createChart(type).init();
  });

  $(function () {


    //for each chart type bind it's control elements with it
    _.each(LS.chart.types, function(type){
      if (LS.chart.disabledControls && LS.chart.disabledControls[type]) {
        return;
      }
      var legendItem = $("#legend-" + type + " .ls-chart-legend-item");
      legendItem.addClass("ls-chart-legend-control");
      legendItem.click(function () {
        var id = $(this).data("id");
        charts[type].toggleTokenChart(id);
      });

      $("#interval-" + type).change(function () {
        charts[type].interval = parseInt($(this).val());
        charts[type].reset();
      });

      var calendarEnterListenerAdded = false;

      var to = new Date(charts[type].to);
      var from = new Date(charts[type].from);
      $('#date-range-field-' + type).daterangepicker({
        startDate: from,
        endDate: to,
        opens: "left",
        maxDate: new Date(),
        autoUpdateInput: true,
        autoApply: true,
        locale: {
          format: 'DD MMMM, YYYY',
          firstDay: 1
        }
      }, function(start, end) {
        var chart = charts[type];
        chart.from = start.toDate();
        chart.fromTXT  = start.format('YYYY-MM-DD');
        chart.to = end.toDate();
        chart.toTXT = end.format('YYYY-MM-DD');
        chart.reset();
      })
      .on('showCalendar.daterangepicker', function(ev, picker) {
        if (calendarEnterListenerAdded) {
          return;
        }
        calendarEnterListenerAdded = true;

        picker.container.find('input').on('keydown.daterangepicker', function(e){
          // enter pressed
          if (e.keyCode === 13) {
            picker.formInputsChanged(e);
            picker.clickApply(e);
          }
        });
      });

      $("#downloadCsv-" + type).click(function(){
        charts[type].downloadCsv();
      });
    });
  });
})();


